// 定义一个类
class timerUtils {
  static errorLog = []
  static init(mainPath){
    this.mainPath = mainPath
    this.screenImage = MNUtil.getImage(mainPath + `/screen.png`,4)
    this.linkImage = MNUtil.getImage(mainPath + `/link.png`)
    this.homeImage = MNUtil.getImage(mainPath + `/home.png`)
    this.goforwardImage = MNUtil.getImage(mainPath + `/goforward.png`)
    this.gobackImage = MNUtil.getImage(mainPath + `/goback.png`)
    this.reloadImage = MNUtil.getImage(mainPath + `/reload.png`)
    this.stopImage = MNUtil.getImage(mainPath + `/stop.png`)
    this.webappImage = MNUtil.getImage(mainPath + `/webapp.png`)
  }
  /**
   * 
   * @param {string} fullPath 
   * @returns {string}
   */
  static getExtensionFolder(fullPath) {
      // 找到最后一个'/'的位置
      let lastSlashIndex = fullPath.lastIndexOf('/');
      // 从最后一个'/'之后截取字符串，得到文件名
      let fileName = fullPath.substring(0,lastSlashIndex);
      return fileName;
  }
  static showHUD(message,duration=2) {
    let app = Application.sharedInstance()
    app.showHUD(message,app.focusWindow,duration)
  }
  static checkMNUtilsFolder(fullPath){
    let extensionFolder = this.getExtensionFolder(fullPath)
    let folderExist = NSFileManager.defaultManager().fileExistsAtPath(extensionFolder+"/marginnote.extension.mnutils/main.js")
    if (!folderExist) {
      this.showHUD("MN Timer: Please install 'MN Utils' first!",5)
    }
    return folderExist
  }
  static async checkMNUtil(alert = false,delay = 0.01){
    if (typeof MNUtil === 'undefined') {//如果MNUtil未被加载，则执行一次延时，然后再检测一次
      //仅在MNUtil未被完全加载时执行delay
      await this.delay(delay)
      if (typeof MNUtil === 'undefined') {
        if (alert) {
          this.showHUD("MN Timer: Please install 'MN Utils' first!",5)
        }
        return false
      }
    }
    return true
  }
  static async delay (seconds) {
    return new Promise((resolve, reject) => {
      NSTimer.scheduledTimerWithTimeInterval(seconds, false, function () {
        resolve()
      })
    })
  }
  static getOrderText(order) {
    if (order[0] == 4) {
      return 'Order: (Title) + (Excerpt → Comment)'
    }
    let orderNumber = `${order[0]}${order[1]}${order[2]}`
    switch (orderNumber) {
      case "123":
        return 'Order: Title → Excerpt → Comment'
      case "132":
        return 'Order: Title → Comment → Excerpt'
      case "213":
        return 'Order: Excerpt → Title → Comment'
      case "231":
        return 'Order: Excerpt → Comment → Title'
      case "312":
        return 'Order: Comment → Title → Excerpt'
      case "321":
        return 'Order: Comment → Excerpt → Title'
      default:
        return "123";
    }
  }
  static isNSNull(obj){
    return (obj === NSNull.new())
  }
  static shouldPrevent(currentURL,requestURL,type) {
    let firstCheck = Application.sharedInstance().osType === 0 && (type===0 || /^https:\/\/m.inftab.com/.test(currentURL))
    if (firstCheck) {
      let blacklist = ["^https?://www.bilibili.com","^https?://m.bilibili.com","^https?://space.bilibili.com","^https?://t.bilibili.com","^https?://www.wolai.com","^https?://flowus.com","^https?://www.notion.so"]
      if (blacklist.some(url=>RegExp(url).test(requestURL))) {
        return true
      }
    }
    return false
  }
  static getNewLoc(gesture){
    // let locationToMN = gesture.locationInView(MNUtil.studyView)
    let locationToMN = gesture.locationInView(MNUtil.currentWindow)
    if (!gesture.moveDate) {
      gesture.moveDate = 0
    }
    if ((Date.now() - gesture.moveDate) > 100) {
      let translation = gesture.translationInView(MNUtil.currentWindow)
      let locationToBrowser = gesture.locationInView(gesture.view.superview)
      // if (gesture.state !== 3 && Math.abs(translation.y)<20 && Math.abs(translation.x)<20) {
      if (gesture.state === 1) {
        gesture.locationToBrowser = {x:locationToBrowser.x-translation.x,y:locationToBrowser.y-translation.y}
        // MNUtil.showHUD(JSON.stringify(gesture.locationToBrowser))
      }
    }
    // MNUtil.showHUD(JSON.stringify(locationToMN))
    if (locationToMN.x <= 0) {
      locationToMN.x = 0
    }
    if (locationToMN.x > MNUtil.currentWindow.frame.width) {
      locationToMN.x = MNUtil.currentWindow.frame.width
    }
    gesture.moveDate = Date.now()
    // let location = {x:locationToMN.x - self.locationToButton.x-gesture.view.frame.x,y:locationToMN.y -self.locationToButton.y-gesture.view.frame.y}
    let location = {x:locationToMN.x - gesture.locationToBrowser.x,y:locationToMN.y -gesture.locationToBrowser.y}
    if (location.y <= 0) {
      location.y = 0
    }
    if (location.y>=MNUtil.currentWindow.frame.height-15) {
      location.y = MNUtil.currentWindow.frame.height-15
    }
    return location
  }
  /**
   * 
   * @returns {Boolean}
   */
  static checkSubscribe(count = true){
    // return true
    if (typeof subscriptionConfig !== 'undefined') {
      let res = subscriptionConfig.checkSubscribed(count)
      return res
    }else{
      this.showHUD("Please install 'MN Subscription' first!")
      return false
    }
  }
  static checkSender(sender,window){
    return MNUtil.app.checkNotifySenderInWindow(sender, window)
  }
  static checkLogo(){
    if (typeof MNUtil === 'undefined') return false
    if (typeof toolbarConfig !== 'undefined' && toolbarConfig.addonLogos && ("MNTimer" in toolbarConfig.addonLogos) && !toolbarConfig.addonLogos["MNTimer"]) {
        return false
    }
    return true
  }
  static setFrame(controller,x,y,width,height){
    if (typeof x === "object") {
      controller.view.frame = x
    }else{
      controller.view.frame = MNUtil.genFrame(x, y, width, height)
    }
    controller.currentFrame = controller.view.frame
  }
  static genLog(error,source){
    return {error:error.toString(),source:source,time:(new Date(Date.now())).toString()}
  }
  static formatSeconds(seconds) {
    // 计算分钟数
    const minutes = Math.floor(seconds / 60);
    // 计算剩余的秒数
    const remainingSeconds = Math.floor(seconds % 60);

    // 格式化为两位数，不足两位的补零
    const formattedMinutes = String(minutes).padStart(2, '0');
    const formattedSeconds = String(remainingSeconds).padStart(2, '0');

    // 返回格式化后的字符串
    return `${formattedMinutes}:${formattedSeconds}`;
}
  static addErrorLog(error,source,info){
    MNUtil.showHUD("MN Timer Error ("+source+"): "+error)
    if (info) {
      this.errorLog.push({error:error.toString(),source:source,info:info,time:(new Date(Date.now())).toString()})
    }else{
      this.errorLog.push({error:error.toString(),source:source,time:(new Date(Date.now())).toString()})
    }
    MNUtil.copyJSON(this.errorLog)
  }
  static ttsHtml(){
  let html = `<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>MP3 Player</title>
</head>
<body>
    <audio controls id="audioPlayer">
        Your browser does not support the audio element.
    </audio>

    <script>
        var myHeaders = new Headers();
        myHeaders.append("Authorization", "Bearer sk-Z009eI4mw8tmOqgvA598C8B7Eb9a4444821018157bC59fF1");
        myHeaders.append("User-Agent", "Apifox/1.0.0 (https://apifox.com)");
        myHeaders.append("Content-Type", "application/json");

        var raw = JSON.stringify({
           "model": "tts-1",
           "input": "The quick brown fox jumped over the lazy dog.",
           "voice": "alloy"
        });

        var requestOptions = {
           method: 'POST',
           headers: myHeaders,
           body: raw,
           redirect: 'follow'
        };

        fetch("https://chatapi.onechats.top/v1/audio/speech", requestOptions)
           .then(response => response.blob())
           .then(blob => {
               var url = URL.createObjectURL(blob);
               var audioPlayer = document.getElementById('audioPlayer');
               var source = document.createElement('source');
               source.src = url;
               source.type = 'audio/mpeg';
               audioPlayer.appendChild(source);
               audioPlayer.load();
           })
           .catch(error => console.log('error', error));
    </script>
</body>
</html>
`
  return html
  }
  static extractJSONFromMarkdown(markdown) {
    // 使用正则表达式匹配被```JSON```包裹的内容
    const regex = /```JSON([\s\S]*?)```/g;
    const matches = regex.exec(markdown);
    
    // 提取匹配结果中的JSON字符串部分，并去掉多余的空格和换行符
    if (matches && matches[1]) {
        const jsonString = matches[1].trim();
        return jsonString;
    } else {
        return undefined;
    }
  }

  static videoInfo2MD(videoFrameInfo){
    let formatedVideoTime = this.formatSeconds(videoFrameInfo.time)
    if ("p" in videoFrameInfo && videoFrameInfo.p) {
      if (timerConfig.getConfig("timestampDetail")) {
        return `![image.png](${videoFrameInfo.image})\n[\[${formatedVideoTime}\] (${videoFrameInfo.bv}-${videoFrameInfo.p})](marginnote4app://addon/BilibiliExcerpt?videoId=${videoFrameInfo.bv}&t=${videoFrameInfo.time}&p=${videoFrameInfo.p})`
      }else{
        return `![image.png](${videoFrameInfo.image})\n[${formatedVideoTime}](marginnote4app://addon/BilibiliExcerpt?videoId=${videoFrameInfo.bv}&t=${videoFrameInfo.time}&p=${videoFrameInfo.p})`
      }
    }
    if (timerConfig.getConfig("timestampDetail")) {
      return `![image.png](${videoFrameInfo.image})\n[\[${formatedVideoTime}\] (${videoFrameInfo.bv})](marginnote4app://addon/BilibiliExcerpt?videoId=${videoFrameInfo.bv}&t=${videoFrameInfo.time})`
    }else{
      return `![image.png](${videoFrameInfo.image})\n[${formatedVideoTime}](marginnote4app://addon/BilibiliExcerpt?videoId=${videoFrameInfo.bv}&t=${videoFrameInfo.time})`
    }
  }
  static getTargetFrame(popupFrame,arrow){
    var x, y
    let w = (MNUtil.app.osType !== 1) ? 419 : 365, // this.addonController.view.frame.width
      h = 500, // this.addonController.view.frame.height
      fontSize = 15,
      margin = 10,
      padding = 20
    let frame = MNUtil.studyView.bounds
    let W = frame.width
    let H = frame.height
    let X = popupFrame.x
    let Y = popupFrame.y
    let contextMenuWidth = MNUtil.studyMode === 0 ? 225 : 435
    let contextMenuHeight = 35
    let textMenuPadding = 40

    // this.addonController.view.frame.x
    if (w >= contextMenuWidth) {
      if (X - w / 2 - margin <= 0) {
        x = margin;
      } else if (X + w / 2 + margin >= W) {
        x = W - margin - w;
      } else {
        x = X - w / 2;
      }
    } else {
      if (X - contextMenuWidth / 2 - margin <= 0) {
        x = margin + contextMenuWidth / 2 - w / 2;
      } else if (X + contextMenuWidth / 2 + margin >= W) {
        x = W - margin - contextMenuWidth / 2 - w / 2;
      } else {
        x = X - w / 2;
      }
    }

    // this.addonController.view.frame.[y, height]
    if (arrow === 1) {
      let upperBlankHeight = Y - textMenuPadding - fontSize - padding,
        lowerBlankHeight = H - Y - contextMenuHeight - padding;
      if (upperBlankHeight >= lowerBlankHeight) {
        h = (upperBlankHeight >= h) ? h : upperBlankHeight;
        y = upperBlankHeight - h;
      } else {
        y = H - lowerBlankHeight;
        h = (H - y >= h) ? h : H - y;
    // this.appInstance.showHUD('x:'+x+';y:'+Y,this.window,2)
      }
    } else {
      let upperBlankHeight = Y - textMenuPadding - contextMenuHeight - padding,
        lowerBlankHeight = H - Y - fontSize - padding;
      if (upperBlankHeight >= lowerBlankHeight) {
        h = (upperBlankHeight >= h) ? h : upperBlankHeight;
        y = upperBlankHeight - h;
      } else {
        y = H - lowerBlankHeight;
        h = (H - y >= h) ? h : H - y;
      }
    }
    return MNUtil.genFrame(x, y, w, h)
  }

}
class timerConfig{
  static get defaultEntries(){
    return {
      Bing:             { title: '🔍 Bing',           symbol: "🔍", engine: "Bing",     desktop:false, link: "https://www.bing.com/search?q=%s" },
      Baidu:            { title: '🔍 Baidu',          symbol: "🔍", engine: "Baidu",    desktop:false, link: "https://www.baidu.com/s?wd=%s" },
      Zhihu:            { title: '🔍 Zhihu',          symbol: "🔍", engine: "Zhihu",    desktop:false, link: "https://www.zhihu.com/search?type=content&q=%s" },
      Google:           { title: '🔍 Google',         symbol: "🔍", engine: "Google",   desktop:false, link: "https://www.google.com/search?q=%s" },
      BaiduTranslate:   { title: '📔 Baidu',          symbol: "📔", engine: "Baidu",    desktop:false, link: "https://fanyi.baidu.com/#en/zh/%s" },
      Deepl:            { title: '📔 Deepl',          symbol: "📔", engine: "Deepl",    desktop:false, link: "https://www.deepl.com/translator#en/zh/%s" },
      Youdao:           { title: '📔 Youdao',         symbol: "📔", engine: "Youdao",   desktop:false, link: "https://dict.youdao.com/m/result?word=%s&lang=en" },
      GoogleTranslate:  { title: '📔 Google',         symbol: "📔", engine: "Google",   desktop:false, link: "https://translate.google.com/?sl=en&tl=zh-CN&text=%s&op=translate" },
      Thesaurus:        { title: '📔 Thesaurus',      symbol: "📔", engine: "Thesaurus",desktop:false, link: "https://www.thesaurus.com/browse/%s" },
      ChatGLM:          { title: '🤖 智谱清言',        symbol: "🤖", engine: "智谱清言",     desktop:false, link: "https://chatglm.cn/detail?from=apply" },
      Yiyan:            { title: '🤖 文心一言',        symbol: "🤖", engine: "文心一言",     desktop:false, link: "https://yiyan.baidu.com/" },
      ResearchGate:     { title: '🎓 ResearchGate',   symbol: "🎓", engine: "RG",       desktop:false, link: "https://www.researchgate.net/search.Search.html?query=%s" },
      GoogleScholar:    { title: '🎓 Google Scholar', symbol: "🎓", engine: "Scholar",  desktop:false, link: "https://scholar.google.com/scholar?q=%s" }
    }
  }
  static get defaultWebAppEntries(){
    return {
      Bilibili:         { title: '📺 Bilibili',       symbol: "📺", engine: "Bilibili", desktop:true, link: "https://www.bilibili.com" },
      Notion:           { title: '📝 Notion',         symbol: "📝", engine: "Notion",   desktop:false, link: "https://www.notion.so" },
      Wolai:            { title: '📝 Wolai',          symbol: "📝", engine: "Wolai",    desktop:false, link: "https://www.wolai.com" },    
      Craft:            { title: '📝 Craft',          symbol: "📝", engine: "Craft",    desktop:false, link: "https://docs.craft.do" },
      Flomo:            { title: '📝 Flomo',          symbol: "📝", engine: "Flomo",    desktop:false, link: "https://v.flomoapp.com/mine" },
      Flowus:           { title: '📝 Flowus',         symbol: "📝", engine: "Flowus",   desktop:false, link: "https://flowus.cn" },
      Yinian:           { title: '📝 Yinian',         symbol: "📝", engine: "Yinian",   desktop:true, link: "https://www.yinian.pro/note/" },
      Baiduyun:         { title: '☁️ Baiduyun',        symbol: "☁️", engine: "Baidu",    desktop:true, link: "https://pan.baidu.com/disk/main#/index?category=all" },
    }
  }
  static get defaultConfig(){
    return{
      syncNoteId: "",
      autoExport:false,
      autoImport:false,
      lastSyncTime:0,
      modifiedTime:0,
      custom:"screenshot",
      custom2:"copyCurrentURL",
      custom3:"bigbang",
      timestampDetail:true,
      homePage:{
        url:'https://m.inftab.com/',
        desktop:false
      }
    }
  }
  static init(){
    this.config = this.defaultConfig
    this.entries = this.defaultEntries
    this.entrieNames = Object.keys(this.entries);
    this.webAppEntries = this.defaultWebAppEntries
    this.webAppEntrieNames =  Object.keys(this.webAppEntries)
    if (!this.webAppEntrieNames.length) {
      this.webAppEntrieNames = Object.keys(this.webAppEntries)
    }
    // MNUtil
    this.toolbar = false
    this.dynamic = false
    this.engine =  "Bing"
    if (!(this.engine in this.entries)) {
      this.engine = this.entrieNames[0]
    }
    // if (!(this.engine in this.entries)) {
    //   this.engine = this.entrieNames[0]
    // }
    this.searchOrder         = [2,1,3];
    if (!this.searchOrder || !this.searchOrder.length) {
      this.searchOrder = [2,1,3]
    }
  }
  static copy(obj){
    return JSON.parse(JSON.stringify(obj))
  }
  static getByDefault(key,defaultValue) {
    let value = NSUserDefaults.standardUserDefaults().objectForKey(key)
    if (value === undefined) {
      NSUserDefaults.standardUserDefaults().setObjectForKey(defaultValue,key)
      return defaultValue
    }
    return value
  }
  static remove(key){
    NSUserDefaults.standardUserDefaults().removeObjectForKey(key)
  }
  static save(key,ignoreExport = false,synchronize = true){
        // MNUtil.showHUD("save "+key)
    switch (key) {
      case "MNTimer_webAppEntries":
        NSUserDefaults.standardUserDefaults().setObjectForKey(this.webAppEntries,"MNTimer_webAppEntries")
        this.config.modefiedTime = Date.now()
        if (!ignoreExport && this.getConfig("autoExport")) {
          this.export(false)
        }
        break;
      case "MNTimer_webAppEntrieNames":
        NSUserDefaults.standardUserDefaults().setObjectForKey(this.webAppEntrieNames,"MNTimer_webAppEntrieNames")
        this.config.modefiedTime = Date.now()
        if (!ignoreExport && this.getConfig("autoExport")) {
          this.export(false)
        }
        break;
      case "MNTimer_entries":
        NSUserDefaults.standardUserDefaults().setObjectForKey(this.entries,"MNTimer_entries")
        this.config.modefiedTime = Date.now()
        if (!ignoreExport && this.getConfig("autoExport")) {
          this.export(false)
        }
        break;
      case "MNTimer_entrieNames":
        NSUserDefaults.standardUserDefaults().setObjectForKey(this.entrieNames,"MNTimer_entrieNames")
        this.config.modefiedTime = Date.now()
        if (!ignoreExport && this.getConfig("autoExport")) {
          this.export(false)
        }
        break;
      case "MNTimer_searchOrder":
        NSUserDefaults.standardUserDefaults().setObjectForKey(this.searchOrder,"MNTimer_searchOrder")
        this.config.modefiedTime = Date.now()
        if (!ignoreExport && this.getConfig("autoExport")) {
          this.export(false)
        }
        break;
      case "MNTimer_dynamic":
        NSUserDefaults.standardUserDefaults().setObjectForKey(this.dynamic,"MNTimer_dynamic")
        this.config.modefiedTime = Date.now()
        if (!ignoreExport && this.getConfig("autoExport")) {
          this.export(false)
        }
        break;
      case "MNTimer_engine":
        NSUserDefaults.standardUserDefaults().setObjectForKey(this.engine,"MNTimer_engine")
        this.config.modefiedTime = Date.now()
        if (!ignoreExport && this.getConfig("autoExport")) {
          this.export(false)
        }
        break;
      case "MNTimer_toolbar":
        NSUserDefaults.standardUserDefaults().setObjectForKey(this.toolbar,"MNTimer_toolbar")
        this.config.modefiedTime = Date.now()
        if (!ignoreExport && this.getConfig("autoExport")) {
          this.export(false)
        }
        break;
      case "MNTimer_config":
        NSUserDefaults.standardUserDefaults().setObjectForKey(this.config,"MNTimer_config")
        this.config.modefiedTime = Date.now()
        if (!ignoreExport && this.getConfig("autoExport")) {
          this.export(false)
        }
        break;
      default:
        break;
    }
    if (synchronize) {
      NSUserDefaults.standardUserDefaults().synchronize()
    }
  }
  static getConfig(key){
    if (this.config[key] !== undefined) {
      return this.config[key]
    }else{
      return this.defaultConfig[key]
    }
  }
  static getCustomEmoji(index){
    let configName = (index === 1)?"custom":"custom"+index
    switch (this.getConfig(configName)) {
      case "screenshot":
      case "videoFrame2Clipboard":
      case "videoFrame2Editor":
      case "videoFrame2Note":
      case "videoFrame2ChildNote":
      case "videoFrameToComment":
      case "videoFrameToNewNote":
        return "🎬";
      case "forward10s":
        return "⏩";
      case "backward10s":
        return "⏪";
      case "pauseOrPlay":
        return "▶️"
      case "bigbang":
        return "💥"
      case "openNewWindow":
      case "openInNewWindow":
        return "➕";
      case "copyCurrentURL":
      case "copyAsMDLink":
      case "openCopiedURL":
        return "🌐"
      default:
        break;
    }
  }
  static async export(alert = true){
  try {
    
    if (!timerUtils.checkSubscribe(true)) {
      return false
    }
    let noteId = this.getConfig("syncNoteId")
    let lastSyncTime = this.getConfig("lastSyncTime") ?? 0
    let modifiedTime = this.getConfig("modifiedTime") ?? 0
    let latestTime = Math.max(lastSyncTime,modifiedTime)
    let focusNote = MNUtil.getNoteById(noteId)
    if (!focusNote) {
      focusNote = MNNote.getFocusNote()
    }
    if (!focusNote) {
      MNUtil.showHUD("Select a note to export!")
      return
    }
    let modifiedDate = Date.parse(focusNote.modifiedDate ?? focusNote.createDate)
    let confirm = false
    // MNUtil.copy(modifiedDate)
    // MNUtil.copyJSON(focusNote.modifiedDate)
    if (latestTime > modifiedDate) {
      // MNUtil.showHUD("should sync")
      confirm = true
    }else{
      // MNUtil.showHUD("should comfirm")
      if (alert) {
        confirm = await MNUtil.confirm("Newer config from note!\n卡片配置较新！","Overwrite?\n是否覆盖？")
      }
    }
    if (!confirm) {
      return false
    }
    this.config.lastSyncTime = Date.now()+5
    this.config.syncNoteId = focusNote.noteId
    let config = {
      config:this.config,
      entries:this.entries,
      entrieNames:this.entrieNames,
      webAppEntries:this.webAppEntries,
      webAppEntrieNames:this.webAppEntrieNames,
      searchOrder:this.searchOrder,
      dynamic:this.dynamic,
      engine:this.engine,
      toolbar:this.toolbar
    }
    MNUtil.undoGrouping(()=>{
      focusNote.excerptText = "```JSON\n"+JSON.stringify(config,null,2)+"\n```"
      focusNote.noteTitle = "MN Timer Config"
      focusNote.note.excerptTextMarkdown = true
    })
    return true
  } catch (error) {
    timerUtils.addErrorLog(error, "export")
  }
    // MNUtil.copyJSON(config)
  }
  static async import(alert = true){

    if (!timerUtils.checkSubscribe(true)) {
      return false
    }
    let noteId = this.config.syncNoteId
    let lastSyncTime = this.config.lastSyncTime ?? 0
    let modifiedTime = this.config.modifiedTime ?? 0
    let latestTime = Math.max(lastSyncTime,modifiedTime)
    let focusNote = MNUtil.getNoteById(noteId)
    if (!focusNote) {
      focusNote = MNNote.getFocusNote()
    }
    if (focusNote.noteTitle !== "MN Timer Config") {
      MNUtil.showHUD("Invalid note title!")
      return false
    }
    let modifiedDate = Date.parse(focusNote.modifiedDate)
    let confirm = false
    // MNUtil.copy(modifiedDate)
    // MNUtil.copyJSON(focusNote.modifiedDate)
    if (latestTime < modifiedDate) {
      // MNUtil.showHUD("should sync")
      confirm = true
    }else{
      // MNUtil.showHUD("should comfirm")
      if (alert) {
        confirm = await MNUtil.confirm("Older config from note!\n卡片配置较旧！","Overwrite?\n是否覆盖？")
      }
    }
    if (!confirm) {
      return false
    }
    let contentToParse = focusNote.excerptText
    if (/```JSON/.test(contentToParse)) {
      contentToParse = timerUtils.extractJSONFromMarkdown(contentToParse)
    }
    let config = JSON.parse(contentToParse)
    if ("config" in config 
    && "entries" in config 
    && "entrieNames" in config 
    && "webAppEntries" in config 
    && "webAppEntrieNames" in config 
    && "searchOrder" in config
    && "dynamic" in config
    && "engine" in config
    && "toolbar" in config
    ) {
      this.config = config.config
      this.config.lastSyncTime = Date.now()
      this.entries = config.entries
      this.entrieNames = config.entrieNames
      this.webAppEntries = config.webAppEntries
      this.webAppEntrieNames = config.webAppEntrieNames
      this.searchOrder = config.searchOrder
      this.dynamic = config.dynamic
      this.engine = config.engine
      this.toolbar = config.toolbar
      this.save("MNTimer_config",true)
      this.save("MNTimer_webAppEntries",true)
      this.save("MNTimer_webAppEntrieNames",true)
      this.save("MNTimer_entries",true)
      this.save("MNTimer_entrieNames",true)
      this.save("MNTimer_searchOrder",true)
      this.save("MNTimer_dynamic",true)
      this.save("MNTimer_engine",true)
      this.save("MNTimer_toolbar",true)
      return true
    }else{
      MNUtil.showHUD("Invalid config in note!")
      return false
    }
  }
  static async sync(){
 try {
  

    if (!timerUtils.checkSubscribe(true)) {
      return false
    }
    let noteId = this.config.syncNoteId
    let lastSyncTime = this.config.lastSyncTime ?? 0
    let modifiedTime = this.config.modifiedTime ?? 0
    let latestTime = Math.max(lastSyncTime,modifiedTime)
    let focusNote = MNUtil.getNoteById(noteId)
    if (!focusNote) {
      focusNote = MNNote.getFocusNote()
    }
    let isVaildNoteConfig = false
    let noteConfig
    let modifiedDate = Date.parse(focusNote.modifiedDate ?? focusNote.createDate)
    if (focusNote.noteTitle === "MN Timer Config") {
      let contentToParse = focusNote.excerptText
      if (/```JSON/.test(contentToParse)) {
        contentToParse = timerUtils.extractJSONFromMarkdown(contentToParse)
      }
      noteConfig = JSON.parse(contentToParse)
      isVaildNoteConfig = ("config" in noteConfig 
    && "entries" in noteConfig 
    && "entrieNames" in noteConfig 
    && "webAppEntries" in noteConfig 
    && "webAppEntrieNames" in noteConfig 
    && "searchOrder" in noteConfig
    && "dynamic" in noteConfig
    && "engine" in noteConfig
    && "toolbar" in noteConfig
    )
      
    }
    if (latestTime < modifiedDate && isVaildNoteConfig) {
      // MNUtil.showHUD("should import")
      this.config = noteConfig.config
      this.config.lastSyncTime = Date.now()
      this.entries = noteConfig.entries
      this.entrieNames = noteConfig.entrieNames
      this.webAppEntries = noteConfig.webAppEntries
      this.webAppEntrieNames = noteConfig.webAppEntrieNames
      this.searchOrder = noteConfig.searchOrder
      this.dynamic = noteConfig.dynamic
      this.engine = noteConfig.engine
      this.toolbar = noteConfig.toolbar

      this.save("MNTimer_config",true)
      this.save("MNTimer_webAppEntries",true)
      this.save("MNTimer_webAppEntrieNames",true)
      this.save("MNTimer_entries",true)
      this.save("MNTimer_entrieNames",true)
      this.save("MNTimer_searchOrder",true)
      this.save("MNTimer_dynamic",true)
      this.save("MNTimer_engine",true)
      this.save("MNTimer_toolbar",true)
      MNUtil.showHUD("Sync Success (import)!")
      return true
    }else{
      // MNUtil.showHUD("should export")
      this.config.lastSyncTime = Date.now()+5
      this.config.syncNoteId = focusNote.noteId
      let config = {
        config:this.config,
        entries:this.entries,
        entrieNames:this.entrieNames,
        webAppEntries:this.webAppEntries,
        webAppEntrieNames:this.webAppEntrieNames,
        searchOrder:this.searchOrder,
        dynamic:this.dynamic,
        engine:this.engine,
        toolbar:this.toolbar
      }
      MNUtil.undoGrouping(()=>{
        focusNote.excerptText = "```JSON\n"+JSON.stringify(config,null,2)+"\n```"
        focusNote.noteTitle = "MN Timer Config"
        focusNote.note.excerptTextMarkdown = true
      })
      MNUtil.showHUD("Sync Success (export)!")
      return true
    }
 } catch (error) {
  MNUtil.showHUD(error)
 }
 }
}
function strCode(str) {  //获取字符串的字节数
    var count = 0;  //初始化字节数递加变量并获取字符串参数的字符个数
    var cn = [8211, 8212, 8216, 8217, 8220, 8221, 8230, 12289, 12290, 12296, 12297, 12298, 12299, 12300, 12301, 12302, 12303, 12304, 12305, 12308, 12309, 65281, 65288, 65289, 65292, 65294, 65306, 65307, 65311]
    var half = [32, 33, 34, 35, 36, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 58, 59, 60, 61, 62, 63, 64, 91, 92, 93, 94, 95, 96, 123, 124, 125, 126,105,108,8211]
    if (str) {  //如果存在字符串，则执行
        len = str.length; 
        for (var i = 0; i < len; i++) {  //遍历字符串，枚举每个字符
          let charCode = str.charCodeAt(i)
            if (charCode>=65 && charCode<=90) {
              count += 1.5;  //大写
            } else if (half.includes(charCode)) {
              count +=0.45
            } else if (cn.includes(charCode)) {
              count +=0.8
            }else if (charCode > 255) {  //字符编码大于255，说明是双字节字符(即是中文)
                count += 2;  //则累加2个
            }else{
                count++;  //否则递加一次
            }
        }
        return count;  //返回字节数
    } else {
        return 0;  //如果参数为空，则返回0个
    }
}


function getWebJS(id) {
  switch (id) {
    case "updateDeeplOffset":
      return `document.getElementsByClassName("dl_header")[0].style.display="none";
        document.getElementsByClassName("lmt__docTrans-tab-container")[0].style.display="none";
        document.getElementsByClassName("lmt__sides_container")[0].style.margin = 0;
        document.querySelector("#dl_translator").style.cssText = "padding-top: 20px";
        document.getElementsByClassName("lmt__language_container")[0].style.display = "none";
        document.getElementsByClassName("lmt__language_container")[1].style.display = "none";
        document.getElementsByClassName("lmt__target_toolbar")[0].style.display = "none";
        document.querySelector("#dl_cookieBanner").style.display="none";
        document.querySelector("#lmt_quotes_article").style.display="none";
        document.querySelector("#lmt__dict").style.margin = 0;
        document.querySelector("#lmt_pro_ad_container").style.display = "none";
        document.querySelector("body > div.dl_footerV2_container").style.display = "none";`
    case "updateThesaurusOffset":
      return `document.getElementsByTagName("header")[0].style.display = "none"
        document.getElementsByTagName("section")[0].style.display = "none"
        document.getElementsByTagName("section")[6].style.display = "none"
        document.getElementsByTagName("section")[7].style.display = "none"
        document.getElementsByTagName("section")[8].style.display = "none"
        document.getElementsByTagName("section")[9].style.display = "none"
        document.getElementsByTagName("p")[5].style.display = "none"
        document.getElementsByClassName("acw ac-widget-placeholder ac-reset")[0].style.display = "none"`
    case "updateBilibiliOffset":
      return `
      document.getElementsByClassName("v-popover-wrap")[0].style.display = "none";
      document.getElementsByClassName("v-popover-wrap")[1].style.display = "none";
      document.getElementsByClassName("v-popover-wrap")[2].style.display = "none";
      document.getElementsByClassName("v-popover-wrap")[3].style.display = "none";
      document.getElementsByClassName("v-popover-wrap")[4].style.display = "none";
      document.getElementsByClassName("v-popover-wrap")[5].style.display = "none";
      document.getElementsByClassName("v-popover-wrap")[6].style.display = "none";
      document.getElementsByClassName("v-popover-wrap")[8].style.display = "none";
      document.getElementsByClassName("v-popover-wrap")[10].style.display = "none";
      document.getElementsByClassName("v-popover-wrap")[11].style.display = "none";
      document.getElementsByClassName("v-popover-wrap")[13].style.display = "none";
      document.getElementsByClassName("recommended-swipe grid-anchor")[0].style.display = "none";
      `
    default:
      return ""
  }
}


function postNotification(name,userInfo) {
  let focusWindow = Application.sharedInstance().focusWindow
  NSNotificationCenter.defaultCenter().postNotificationNameObjectUserInfo(name, focusWindow, userInfo)
  
}